package org.artifactory.event.priority.service;

import org.artifactory.event.priority.service.model.PrioritizedNodeEventMapper;
import org.artifactory.storage.StorageException;
import org.artifactory.storage.db.InternalDbService;
import org.artifactory.storage.db.event.dao.NodeEventPrioritiesDao;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.sql.SQLException;

/**
 * @author Uriah Levy
 */
@Service
public class NodeEventPriorityStorageServiceImpl implements NodeEventPriorityStorageService {
    private static final Logger log = LoggerFactory.getLogger(NodeEventPriorityStorageServiceImpl.class);
    private PrioritizedNodeEventMapper prioritizedEventMapper;
    private InternalDbService dbService;
    private NodeEventPrioritiesDao eventPrioritiesDao;

    @Autowired
    public NodeEventPriorityStorageServiceImpl(NodeEventPrioritiesDao eventPrioritiesDao,
            PrioritizedNodeEventMapper prioritizedEventMapper, InternalDbService dbService) {
        this.eventPrioritiesDao = eventPrioritiesDao;
        this.prioritizedEventMapper = prioritizedEventMapper;
        this.dbService = dbService;
    }

    // TODO add DB test with SQLException and make sure rolled back
    public void savePriorityChart(NodeEventPriorityChart nodeEventPriorityChart) {
        nodeEventPriorityChart.getEvents().stream()
                .map(event -> prioritizedEventMapper.toDbEvent(event))
                .forEach(event -> {
                    try {
                        log.debug("Saving event priority with path '{}', and type '{}'", event.getPath(),
                                event.getType());
                        event.setPriorityId(dbService.nextId());
                        eventPrioritiesDao.insertEventPriority(event);
                    } catch (SQLException e) {
                        throw new StorageException("Unable to save new event priority", e);
                    } catch (Exception e) {
                        TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    }
                });
    }
}
