package org.artifactory.event.provider;

import com.google.common.collect.Iterables;
import org.artifactory.storage.db.event.model.NodeEventCursor;
import org.artifactory.storage.event.EventInfo;
import org.artifactory.storage.event.EventsService;

import java.util.Collections;
import java.util.List;

/**
 * @author Uriah Levy
 * An event provider that reads from the event_log table
 */
public class EventLogTableEventProvider implements EventProvider<EventInfo> {

    private EventsService eventsService;

    public EventLogTableEventProvider(EventsService eventsService) {
        this.eventsService = eventsService;
    }

    @Override
    public List<EventInfo> provideNextBatch(NodeEventCursor currentCursor) {
        // TODO [by U.L] limit the number of results. risky
        List<EventInfo> events = eventsService.getEventsAfter(currentCursor.getEventMarker());
        if (!events.isEmpty()) {
            List<EventInfo> eventsWithIdenticalTimestamp = eventsService
                    .getEventsByTimestamp(Iterables.getLast(events).getTimestamp());
            eventsWithIdenticalTimestamp.stream()
                    .filter(event -> !events.contains(event))
                    .forEach(events::add);
            return events;
        }
        return Collections.emptyList();
    }

    @Override
    public ProviderType getType() {
        return ProviderType.EVENT_LOG;
    }
}
