package org.artifactory.metadata.service.provider;

import com.google.common.collect.ImmutableMap;
import lombok.Data;
import org.artifactory.aql.api.domain.sensitive.AqlApiItem;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.repo.RepoPath;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.function.Function;

import static org.artifactory.aql.api.domain.sensitive.AqlApiItem.*;
import static org.artifactory.aql.api.internal.AqlBase.and;

/**
 * @author Uriah Levy
 */
@Component
public class DockerMetadataProvider extends AbstractMetadataProvider {

    private static final RepoType type = RepoType.Docker;

    public DockerMetadataProvider() {
        super(type, DockerMetadataProvider::buildVersionFilesQuery);
        setTypeSpecificMetadataTemplate(new DockerTypeSpecificMetadataTemplate());
    }

    @Override
    public RepoType getRepoType() {
        return type;
    }

    @Override
    public boolean isLeadArtifact(RepoPath repoPath) {
        return repoPath.getName().equals("manifest.json");
    }

    private static AqlApiItem buildVersionFilesQuery(RepoPath repoPath) {
        if (repoPath.getParent() == null) {
            // Root is not supported.
            return null;
        }
        return create()
                .filter(
                        and(
                                repo().equal(repoPath.getRepoKey()),
                                path().equal(repoPath.getParent().getPath()))
                );
    }

    @Data
    class DockerTypeSpecificMetadataTemplate implements TypeSpecificMetadataTemplate {
        private static final String NAME_SUFFIX = ".repoName";
        private static final String VERSION_SUFFIX = ".manifest";
        private Map<String, String> qualifierKeys = ImmutableMap
                .of("docker.manifest.type", "manifest_version", "docker.manifest.digest", "manifest_digest");

        DockerTypeSpecificMetadataTemplate() {
        }

        @Override
        public String getNameKey() {
            return type.getType() + NAME_SUFFIX;
        }

        @Override
        public String getVersionKey() {
            return type.getType() + VERSION_SUFFIX;
        }

        @Override
        public Map<String, String> getQualifierKeys() {
            return qualifierKeys;
        }

        @Override
        public Function<String, String> getQualifierValueResolver() {
            return Function.identity();
        }
    }
}
