package org.artifactory.metadata.service.provider;

import com.google.common.collect.ImmutableMap;
import lombok.Data;
import org.apache.commons.lang.StringUtils;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.repo.RepoPath;
import org.jfrog.client.util.PathUtils;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.function.Function;

/**
 * @author Uriah Levy
 */
@Component
class NpmMetadataProvider extends AbstractMetadataProvider {

    private static final RepoType type = RepoType.Npm;

    public NpmMetadataProvider() {
        super(type, null);
        setTypeSpecificMetadataTemplate(new NpmTypeSpecificMetadataTemplate());
    }

    @Override
    public RepoType getRepoType() {
        return type;
    }

    @Override
    public boolean isLeadArtifact(RepoPath repoPath) {
        String extension = PathUtils.getExtension(repoPath.getPath());
        return StringUtils.isNotBlank(extension) && extension.equals("tgz");
    }

    @Data
    class NpmTypeSpecificMetadataTemplate implements TypeSpecificMetadataTemplate {
        private static final String NAME_SUFFIX = ".name";
        private static final String VERSION_SUFFIX = ".version";
        private Function<String, String> qualifierValueResolver;
        private Map<String, String> qualifiersKeys = ImmutableMap.of("npm.name", "npm_scope");

        NpmTypeSpecificMetadataTemplate() {
        }

        @Override
        public String getNameKey() {
            return type.getType() + NAME_SUFFIX;
        }

        @Override
        public String getVersionKey() {
            return type.getType() + VERSION_SUFFIX;
        }

        @Override
        public Map<String, String> getQualifierKeys() {
            return qualifiersKeys;
        }

        @Override
        public Function<String, String> getQualifierValueResolver() {
            return rawValue -> {
                if (StringUtils.isNotBlank(rawValue) && rawValue.startsWith("@")) {
                    // "@scope/pkg" -> "scope"
                    return StringUtils.substringBefore(rawValue, "/").substring(1);
                }
                return "";
            };
        }
    }
}
