package org.artifactory.metadata.service.provider;

import com.google.common.collect.ImmutableMap;
import lombok.Data;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.repo.RepoPath;
import org.jfrog.client.util.PathUtils;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.function.Function;

/**
 * @author Uriah Levy
 */
@Component
public class RpmMetadataProvider extends AbstractMetadataProvider {
    private static final RepoType type = RepoType.YUM;

    public RpmMetadataProvider() {
        super(type, null);
        setTypeSpecificMetadataTemplate(new RpmTypeSpecificMetadataTemplate());
    }

    @Override
    public RepoType getRepoType() {
        return type;
    }

    @Override
    public boolean isLeadArtifact(RepoPath repoPath) {
        return PathUtils.getExtension(repoPath.getPath()).equals("rpm");
    }

    @Data
    class RpmTypeSpecificMetadataTemplate implements TypeSpecificMetadataTemplate {
        private static final String METADATA_PREFIX = "rpm.metadata";
        private static final String NAME_SUFFIX = ".name";
        private static final String VERSION_SUFFIX = ".version";
        private final Map<String, String> qualifierKeys = ImmutableMap
                .of(METADATA_PREFIX + ".arch", "rpm_architecture", METADATA_PREFIX + ".group", "rpm_group");

        RpmTypeSpecificMetadataTemplate() {
        }

        @Override
        public String getNameKey() {
            return METADATA_PREFIX + NAME_SUFFIX;
        }

        @Override
        public String getVersionKey() {
            return METADATA_PREFIX + VERSION_SUFFIX;
        }

        @Override
        public Map<String, String> getQualifierKeys() {
            return qualifierKeys;
        }

        @Override
        public Function<String, String> getQualifierValueResolver() {
            return Function.identity();
        }
    }
}
