/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.repo;

import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.ProtocolException;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.DefaultRedirectStrategy;
import org.apache.http.protocol.HttpContext;
import org.apache.http.util.TextUtils;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.Locale;

/**
 * Redirect strategy that supports 308 Permanent Redirect status code
 *
 * @author nadavy
 */
public class JFrogRedirectStrategy extends DefaultRedirectStrategy {

    private static final int REDIRECT_PERMANENTLY = 308;
    private boolean isNeedToNormalize;
    private String[] queryParamsArray;

    JFrogRedirectStrategy(boolean isNeedToNormalize, String queryParams) {
        this.isNeedToNormalize = isNeedToNormalize;
        this.queryParamsArray = StringUtils.split(queryParams, ",");
    }


    @Override
    public boolean isRedirected(HttpRequest request, HttpResponse response, HttpContext context)
            throws ProtocolException {
        return super.isRedirected(request, response, context) ||
                (response.getStatusLine().getStatusCode() == REDIRECT_PERMANENTLY &&
                        this.isRedirectable(request.getRequestLine().getMethod()));
    }

    @Override
    protected URI createLocationURI(String location) throws ProtocolException {
        if (isNeedToNormalize || !isLocationContainsSignUrlQueryParam(location)) {
            return super.createLocationURI(location);
        } else {
            try {
                URIBuilder b = new URIBuilder((new URI(location)));
                String host = b.getHost();
                if (host != null) {
                    b.setHost(host.toLowerCase(Locale.ROOT));
                }
                String path = b.getPath();
                if (TextUtils.isEmpty(path)) {
                    b.setPath("/");
                }
                return b.build();
            } catch (URISyntaxException var5) {
                throw new ProtocolException("Invalid redirect URI: " + location, var5);
            }
        }
    }

    private boolean isLocationContainsSignUrlQueryParam(String location) {
        return Arrays.stream(queryParamsArray).anyMatch(param -> location.contains(param + "="));
    }
}
