package org.artifactory.repo;

import org.apache.http.ProtocolException;
import org.testng.annotations.Test;

import java.net.URI;

import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

/**
 * @author Lior Gur
 */
@Test

public class JFrogRedirectStrategyTest {

    private String uriWithDoubleSlashWithSig = "https://mcrweu0.cdn.mscr.io/42012bb2682a4d76ba7fa17a9d9a9162-qb2vm9uiex//docker/registry/v2/blobs/sha256/bc/bc2b6cd40cb975fab7f9a8c67d517c0e3387a6cdd234875c52c6a1c959392f88/data?P1=1552393876&P2=1&P3=1&P4=hz2E42l%2Fr9vg5xKwPYAJOQPf2ZKmHdXEKx3cqrzcmbg%3D&se=2019-03-12T12%3A31%3A16Z&sig=KHZ8aXjS1PX9C%2BMHC4f89J4Z9%2FhWSESb6GQuQiRPaiM%3D&sp=r&sr=b&sv=2016-05-31&regid=42012bb2682a4d76ba7fa17a9d9a9162";
    private String uriWithDoubleSlashWithoutSig = "https://mcrweu0.cdn.mscr.io/42012bb2682a4d76ba7fa17a9d9a9162-qb2vm9uiex//docker/registry/v2/blobs/sha256/bc/bc2b6cd40cb975fab7f9a8c67d517c0e3387a6cdd234875c52c6a1c959392f88/data?P1=1552393876&P2=1&P3=1&P4=hz2E42l%2Fr9vg5xKwPYAJOQPf2ZKmHdXEKx3cqrzcmbg%3D&se=2019-03-12T12%3A31%3A16Z&sp=r&sr=b&sv=2016-05-31&regid=42012bb2682a4d76ba7fa17a9d9a9162";

    public void testUriWithDoubleSlashWithSigNoNeedNormalize() throws ProtocolException {
        JFrogRedirectStrategy strategy = new JFrogRedirectStrategy(false, "sig");
        URI locationURI = strategy.createLocationURI(uriWithDoubleSlashWithSig);
        assertTrue(locationURI.toString().contains("//docker"));

        strategy = new JFrogRedirectStrategy(false, "sig212121");
        locationURI = strategy.createLocationURI(uriWithDoubleSlashWithSig);
        assertFalse(locationURI.toString().contains("//docker"));
    }

    public void testUriWithDoubleSlashWithoutSigNeedNoNormalize() throws ProtocolException {
        JFrogRedirectStrategy strategy = new JFrogRedirectStrategy(false, "sig");
        URI locationURI = strategy.createLocationURI(uriWithDoubleSlashWithoutSig);
        assertFalse(locationURI.toString().contains("//docker"));

        strategy = new JFrogRedirectStrategy(false, "sig212121");
        locationURI = strategy.createLocationURI(uriWithDoubleSlashWithoutSig);
        assertFalse(locationURI.toString().contains("//docker"));
    }

    public void testUriWithDoubleSlashWithSigNeedNormalize() throws ProtocolException {
        JFrogRedirectStrategy strategy = new JFrogRedirectStrategy(true, "sig");
        URI locationURI = strategy.createLocationURI(uriWithDoubleSlashWithSig);
        assertFalse(locationURI.toString().contains("//docker"));

        strategy = new JFrogRedirectStrategy(true, "sig212121");
        locationURI = strategy.createLocationURI(uriWithDoubleSlashWithSig);
        assertFalse(locationURI.toString().contains("//docker"));
    }

    public void testUriWithDoubleSlashWithoutSigNeedNormalize() throws ProtocolException {
        JFrogRedirectStrategy strategy = new JFrogRedirectStrategy(true, "sig");
        URI locationURI = strategy.createLocationURI(uriWithDoubleSlashWithoutSig);
        assertFalse(locationURI.toString().contains("//docker"));

        strategy = new JFrogRedirectStrategy(true, "sig212121");
        locationURI = strategy.createLocationURI(uriWithDoubleSlashWithoutSig);
        assertFalse(locationURI.toString().contains("//docker"));
    }



}