package org.artifactory.repo.service.trash;

import org.artifactory.common.ArtifactoryHome;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import static org.testng.Assert.assertEquals;

/**
 * @author dudim
 * Retention days cannot exceed 5 years value.
 * when executing propertiesDao.getGCCandidateQuery we using less('<') comparator on strings (prop_value < validfrom).
 * we assume validFrom is with length of 10 chars (since 2002 epoch time length is 10 digits),
 * in this test we assert when setting the Retention days to over 5 years(1825 days) we will return 0
 * otherwise we will delete all trashcan content.
 */
@Test
public class TrashServiceImplTest extends ArtifactoryHomeBoundTest {

    @Test(dataProvider = "isBatchCleanup")
    public void testNormalizeRetentionPeriodDaysZeroValue(boolean isBatchCleanup) throws Exception {
        setBatchCleanup(isBatchCleanup);
        TrashServiceImpl trashService = new TrashServiceImpl();
        assertEquals(trashService.normalizeRetentionDays(0), 0);
    }

    @Test(dataProvider = "isBatchCleanup")
    public void testNormalizeRetentionPeriodDaysValidValue(boolean isBatchCleanup) throws Exception {
        setBatchCleanup(isBatchCleanup);
        TrashServiceImpl trashService = new TrashServiceImpl();
        assertEquals(trashService.normalizeRetentionDays(15), 15);
    }

    @Test
    public void testNormalizeRetentionPeriodDaysOver5Years() throws Exception {
        setBatchCleanup(true);
        TrashServiceImpl trashService = new TrashServiceImpl();
        assertEquals(trashService.normalizeRetentionDays(1900), 0);
    }

    @Test
    public void testNormalizeRetentionPeriodDaysOver5YearsWithCleanupBatchFalse() throws Exception {
        setBatchCleanup(false);
        TrashServiceImpl trashService = new TrashServiceImpl();
        assertEquals(trashService.normalizeRetentionDays(2900), 2900);
    }

    private void setBatchCleanup(boolean isEnable) {
        ArtifactoryHome.get()
                .getArtifactoryProperties()
                .setProperty("artifactory.gc.batch.cleanup", String.valueOf(isEnable));
    }

    @DataProvider
    public Object[][] isBatchCleanup() {
        return new Object[][]{{true}, {false}};
    }
}