/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.security.access;

import org.jfrog.access.common.ServiceId;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import static org.artifactory.security.access.AccessServiceConstants.ARTIFACTORY_SERVICE_TYPE;
import static org.testng.Assert.assertEquals;

/**
 * @author Yinon Avraham.
 */
public class ArtifactoryAdminScopeTokenTest {

    @Test(dataProvider = "provideParse")
    public void testParse(String scopeToken, ServiceId serviceId) throws Exception {
        ArtifactoryAdminScopeToken adminScopeToken = ArtifactoryAdminScopeToken.parse(scopeToken);
        assertEquals(adminScopeToken.getServiceId(), serviceId);
        assertEquals(adminScopeToken.getScopeToken(), scopeToken);
        assertEquals(adminScopeToken.toString(), scopeToken);
    }

    @DataProvider
    public static Object[][] provideParse() {
        ServiceId artServiceId1 = ServiceId.generateUniqueId(ARTIFACTORY_SERVICE_TYPE);
        ServiceId artServiceId2 = ServiceId.generateUniqueId(ARTIFACTORY_SERVICE_TYPE);
        ServiceId artServiceId3 = ServiceId.generateUniqueId("jf-artifactory");
        return new Object[][]{
                { artServiceId1 + ":admin", artServiceId1 },
                { artServiceId2 + ":admin", artServiceId2 },
                { artServiceId3 + ":admin", artServiceId3 }
        };
    }

    @Test(dataProvider = "provideAccepts")
    public void testAccepts(String scopeToken, boolean expected) throws Exception {
        assertEquals(ArtifactoryAdminScopeToken.accepts(scopeToken), expected);
    }

    @DataProvider
    public static Object[][] provideAccepts() {
        ServiceId artServiceId1 = ServiceId.generateUniqueId(ARTIFACTORY_SERVICE_TYPE);
        return new Object[][]{
                { ServiceId.generateUniqueId("jf-artifactory") + ":admin", true },
                { artServiceId1 + ":admin", true },
                { artServiceId1 + ":admin ", false },
                { " " + artServiceId1 + ":admin", false },
                { artServiceId1 + ":admins", false },
                { artServiceId1 + ":", false },
                { artServiceId1 + "", false },
                { ServiceId.generateUniqueId("other-service-type") + ":admin", false }
        };
    }
}