/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.traffic;

import org.apache.commons.lang.StringUtils;
import org.artifactory.traffic.entry.TransferEntry;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Logger class for traffic, filtering events made by Xray
 *
 * @author Tamir Hadad
 */
abstract class XrayFilteredTrafficLogger {
    private static final Logger log = LoggerFactory.getLogger(XrayFilteredTrafficLogger.class);

    private XrayFilteredTrafficLogger() {
        // utility class
    }

    /**
     * Log without Xray traffic
     *
     * @param entry Traffic entry to log
     */
    static void logTransferEntry(TransferEntry entry) {
        String userIdentifier = entry.getUserIdentifier();
        if (StringUtils.isEmpty(userIdentifier) || !userIdentifier.toLowerCase().contains("xray")) {
            log.info(entry.toString());
        }
    }
}