/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.util;

import org.artifactory.version.ArtifactoryVersion;
import org.testng.annotations.Test;

import static org.testng.Assert.assertEquals;

/**
 * @author Dan Feldman
 */
public class ArtifactoryNodeVersionComparatorTest {

    private static final String gingiVersion = "5.4.0";
    private final ArtifactoryNodeVersionComparator comparator = new ArtifactoryNodeVersionComparator();

    @Test
    public void testIsVersionEqual() {
        for (ArtifactoryVersion artifactoryVersion : ArtifactoryVersion.values()) {
            String stringVersion = artifactoryVersion.getValue();
            boolean actual = comparator.isVersionEqual(stringVersion, gingiVersion);
            boolean expected = artifactoryVersion.equals(ArtifactoryVersion.fromVersionString(gingiVersion));
            assertEquals(actual, expected);
        }
    }

    @Test
    public void testIsVersionBefore() {
        for (ArtifactoryVersion artifactoryVersion : ArtifactoryVersion.values()) {
            String stringVersion = artifactoryVersion.getValue();
            boolean actual = comparator.isVersionBefore(stringVersion, gingiVersion);
            boolean expected = artifactoryVersion.before(ArtifactoryVersion.fromVersionString(gingiVersion));
            assertEquals(actual, expected);
        }
    }

    @Test
    public void testIsVersionAfter() {
        for (ArtifactoryVersion artifactoryVersion : ArtifactoryVersion.values()) {
            String stringVersion = artifactoryVersion.getValue();
            boolean actual = comparator.isVersionAfter(stringVersion, gingiVersion);
            boolean expected = artifactoryVersion.after(ArtifactoryVersion.fromVersionString(gingiVersion));
            assertEquals(actual, expected);
        }
    }
}