package org.artifactory.storage.binstore.service;

import com.google.common.collect.ImmutableList;
import org.artifactory.common.ArtifactoryHome;
import org.artifactory.common.ConstantValues;
import org.artifactory.common.property.ArtifactorySystemProperties;
import org.artifactory.storage.GCCandidate;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.testng.annotations.Test;

import static org.mockito.Mockito.*;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

/**
 * @author dudim
 */
@Test
public class GCProviderWrapperTest extends ArtifactoryHomeBoundTest {

    public void testErrorCount() {
        ArtifactorySystemProperties artifactoryProperties = ArtifactoryHome.get().getArtifactoryProperties();
        artifactoryProperties.setProperty(ConstantValues.gcFailCountThreshold.getPropertyName(), "3");
        GCProvider gcProviderMock = mock(GCProvider.class);

        doNothing().when(gcProviderMock).resetProvider();
        when(gcProviderMock.getBatch()).thenReturn(ImmutableList.of(
                new GCCandidate(null, "isEnabled=true", null, null, 0)));

        when(gcProviderMock.getAction()).thenReturn((o, o2) -> {
            System.out.println("Action is performed");
            throw new RuntimeException("increasing error count");
        });

        GCProviderWrapper gcProviderWrapper = new GCProviderWrapper(gcProviderMock);
        gcProviderWrapper.resetProvider();
        assertEquals(gcProviderWrapper.getBatch().get(0).getSha1(), "isEnabled=true");

        gcProviderWrapper.getAction()
                .accept(new GCCandidate(null, null, null, null, 0), new GarbageCollectorInfo());
        assertEquals(gcProviderWrapper.getBatch().get(0).getSha1(), "isEnabled=true");

        gcProviderWrapper.getAction()
                .accept(new GCCandidate(null, null, null, null, 0), new GarbageCollectorInfo());
        assertEquals(gcProviderWrapper.getBatch().get(0).getSha1(), "isEnabled=true");

        gcProviderWrapper.getAction()
                .accept(new GCCandidate(null, null, null, null, 0), new GarbageCollectorInfo());
        assertTrue(gcProviderWrapper.getBatch().isEmpty());
    }
}