package org.artifactory.storage.db.binstore.service.garbage;

import org.artifactory.api.repo.RepositoryService;
import org.artifactory.common.ConstantValues;
import org.artifactory.storage.GCCandidate;
import org.artifactory.storage.binstore.service.BinaryService;
import org.artifactory.storage.binstore.service.GCProvider;
import org.artifactory.storage.binstore.service.GarbageCollectorInfo;
import org.artifactory.storage.db.binstore.dao.BinariesDao;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.function.BiConsumer;

/**
 * @author dudim
 */
public class BinariesGCProvider implements GCProvider {
    private static final Logger log = LoggerFactory.getLogger(TrashGCProvider.class);

    private BinaryService binaryService;
    private RepositoryService repositoryService;
    private GCStreamingQueryHelper gcStreamingQueryHelper;

    public BinariesGCProvider(BinaryService binaryService,
            BinariesDao binariesDao, RepositoryService repositoryService) {
        this.binaryService = binaryService;
        this.repositoryService = repositoryService;
        gcStreamingQueryHelper = new GCStreamingQueryHelper(ConstantValues.trashcanMaxSearchResults.getInt(),
                binaryService::getBinariesGCCandidatesStreaming, binariesDao::gcCandidateFromResultSet);
    }

    @Override
    public List<GCCandidate> getBatch() {
        return gcStreamingQueryHelper.nextBatch();
    }

    @Override
    public BiConsumer<GCCandidate, GarbageCollectorInfo> getAction() {
        return (gcCandidate, result) -> {
            if (!ConstantValues.gcBinariesJoinWithNodesQuery.getBoolean() &&
                    repositoryService.existsBySha1(gcCandidate.getSha1())) {
                log.info("Binary: {} exist, will not be deleted", gcCandidate.getSha1());
                return;
            }
            binaryService.executeBinaryCleaner(gcCandidate, result);
        };
    }

    @Override
    public void resetProvider() {
        gcStreamingQueryHelper.reset();
    }

    @Override
    public boolean reportAfterBatch() {
        return true;
    }
}
