package org.artifactory.storage.db.binstore.service.garbage;

import org.artifactory.api.repo.RepositoryService;
import org.artifactory.repo.trash.TrashService;
import org.artifactory.storage.GCCandidate;
import org.artifactory.storage.binstore.service.BinaryService;
import org.artifactory.storage.binstore.service.GCProvider;
import org.artifactory.storage.binstore.service.GarbageCollectorInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.function.BiConsumer;

/**
 * @author dudim
 */
public class TrashAndBinariesGCProvider implements GCProvider {

    private static final Logger log = LoggerFactory.getLogger(TrashAndBinariesGCProvider.class);

    private BinaryService binaryService;
    private RepositoryService repoService;
    private TrashService trashService;

    public TrashAndBinariesGCProvider(RepositoryService repoService, TrashService trashService,
            BinaryService binaryService) {
        this.trashService = trashService;
        this.repoService = repoService;
        this.binaryService = binaryService;
    }

    @Override
    public List<GCCandidate> getBatch() {
        List<GCCandidate> trashCandidates = trashService.getGCCandidatesFromTrash();
        log.debug("Found {} GC candidates from Trash", trashCandidates.size());
        return new ArrayList<>(trashCandidates);
    }

    @Override
    public BiConsumer<GCCandidate, GarbageCollectorInfo> getAction() {
        return (gcCandidate, result) -> {
            trashService.undeployFromTrash(gcCandidate);
            if (!repoService.existsBySha1(gcCandidate.getSha1())) {
                binaryService.executeBinaryCleaner(gcCandidate, result);
            }
        };
    }

    @Override
    public void resetProvider() {
        // noop
    }

    @Override
    public boolean reportAfterBatch() {
        return true;
    }
}
