package org.artifactory.storage.db.binstore.service.garbage;

import org.artifactory.common.ConstantValues;
import org.artifactory.repo.trash.TrashService;
import org.artifactory.storage.GCCandidate;
import org.artifactory.storage.binstore.service.GCProvider;
import org.artifactory.storage.binstore.service.GarbageCollectorInfo;
import org.artifactory.storage.db.fs.dao.NodesDao;

import java.util.List;
import java.util.function.BiConsumer;

/**
 * @author dudim
 */
public class TrashGCProvider implements GCProvider {
    private GCStreamingQueryHelper gcStreamingQueryHelper;
    private TrashService trashService;

    public TrashGCProvider(TrashService trashService, NodesDao nodesDao) {
        this.trashService = trashService;
        gcStreamingQueryHelper = new GCStreamingQueryHelper(ConstantValues.trashcanMaxSearchResults.getInt(),
                trashService::getGCCandidatesFromTrashStreaming, nodesDao::gcCandidateFromResultSet);
    }

    @Override
    public List<GCCandidate> getBatch() {
        return gcStreamingQueryHelper.nextBatch();
    }

    @Override
    public BiConsumer<GCCandidate, GarbageCollectorInfo> getAction() {
        return (gcCandidate, result) -> trashService.undeployFromTrash(gcCandidate);
    }

    @Override
    public void resetProvider() {
        gcStreamingQueryHelper.reset();
    }

    @Override
    public boolean reportAfterBatch() {
        return false;
    }
}
