package org.artifactory.storage.db.event.entity;

import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.jfrog.common.mapper.Validatable;

import static org.jfrog.common.ArgUtils.requireNonBlank;
import static org.jfrog.common.ArgUtils.requireNonNegative;

/**
 * @author Uriah Levy
 */
@Data
@NoArgsConstructor
@Builder(toBuilder = true, builderClassName = "Builder")
public class DbNodeEventCursor implements Validatable {

    private String operatorId;
    private long eventMarker;

    public DbNodeEventCursor(String operatorId, long eventMarker) {
        validate(operatorId, eventMarker);
    }

    @Override
    public void validate() {
        validate(operatorId, eventMarker);
    }

    public void validate(String operationId, long eventTimestamp) {
        this.operatorId = requireNonBlank(operationId, "Operation ID is required");
        this.eventMarker = requireNonNegative(eventTimestamp, "Event Marker is required");
    }
}
