package org.artifactory.storage.db.event.service;

import org.artifactory.storage.StorageException;
import org.artifactory.storage.db.event.dao.NodeEventsCursorDao;
import org.artifactory.storage.db.event.entity.DbNodeEventCursor;
import org.artifactory.storage.db.event.model.NodeEventCursor;
import org.artifactory.storage.db.event.service.mapper.NodeEventCursorMapper;
import org.artifactory.storage.db.event.service.mapper.NodeEventCursorMapperImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.SQLException;
import java.util.Optional;

/**
 * @author Uriah Levy
 */
@Service
public class NodeEventCursorServiceImpl implements NodeEventCursorService {
    private static final Logger log = LoggerFactory.getLogger(NodeEventCursorServiceImpl.class);

    private NodeEventsCursorDao nodeEventsCursorDao;
    private NodeEventCursorMapper nodeEventCursorMapper;

    @Autowired
    NodeEventCursorServiceImpl(NodeEventsCursorDao nodeEventsCursorDao) {
        this.nodeEventsCursorDao = nodeEventsCursorDao;
        this.nodeEventCursorMapper = new NodeEventCursorMapperImpl();
    }

    @Override
    public void updateCursor(NodeEventCursor eventCursor) {
        try {
            log.info("Updating cursor for operator '{}' with timestamp {}", eventCursor.getOperatorId(),
                    eventCursor.getEventMarker());
            nodeEventsCursorDao.update(nodeEventCursorMapper.nodeEventCursorToDbNodeEventCursor(eventCursor));
        } catch (SQLException e) {
            throw new StorageException("Could not update cursor for operator with ID '" + eventCursor.getOperatorId() +
                    "'", e);
        }
    }

    @Override
    public void insertCursor(String operatorId) {
        try {
            nodeEventsCursorDao.insert(nodeEventCursorMapper
                    .nodeEventCursorToDbNodeEventCursor(new NodeEventCursor(operatorId, 0)));
        } catch (SQLException e) {
            throw new StorageException("Could not insert cursor for operator with ID '" + operatorId + "'", e);
        }
    }

    @Override
    public NodeEventCursor cursorForOperator(String operatorId) {
        try {
            Optional<DbNodeEventCursor> dbNodeEventCursorOptional = nodeEventsCursorDao.cursorForOperator(operatorId);
            return dbNodeEventCursorOptional
                    .map(value -> nodeEventCursorMapper.dbNodeEventCursorToNodeEventCursor(value))
                    .orElse(null);
        } catch (SQLException e) {
            throw new StorageException("Unable to retrieve cursor for operator with ID " + operatorId, e);

        }
    }
}