package org.artifactory.storage.db.event.service.metadata.mapper;

import org.jfrog.metadata.client.model.*;
import org.mapstruct.Mapper;
import org.mapstruct.Named;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Uriah Levy
 */
@Mapper(componentModel = "spring")
@Named("MdsEntityTranslator")
class MdsEntityTranslator {

    @Named("LicensesTranslator")
    List<MdsLicense> licensesNamesToMdsLicense(List<String> licenses) {
        // TODO real mds model values
        if (!licenses.isEmpty()) {
            return licenses.stream()
                    .map(licenseName -> new MdsLicense(licenseName, null, null, 0, null))
                    .collect(Collectors.toList());
        }
        return null;
    }

    @Named("StatsTranslator")
    MdsVersionStatsEntity downloadCountToMdsPackageStatsEntity(long downloadCount) {
        return new MdsVersionStatsEntity(downloadCount);
    }

    @Named("UserPropertiesTranslator")
    List<MdsUserProperty> propertyMapToMdsUserPropertyList(Map<String, Set<String>> propertyMap) {
        if (!propertyMap.isEmpty()) {
            return propertyMap.entrySet().stream()
                    .map(entry -> new MdsUserProperty(entry.getKey(), String.join(",", entry.getValue())))
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    @Named("QualifiersTranslator")
    List<MdsQualifierEntity> createMdsQualifiers(Map<String, String> qualifiersMap) {
        if (!qualifiersMap.isEmpty()) {
            return qualifiersMap.entrySet().stream()
                    .map(entry -> new MdsQualifierEntity(entry.getKey(), entry.getValue()))
                    .collect(Collectors.toList());
        }
        return Collections.emptyList();
    }
}
