package org.artifactory.storage.db.event.service.metadata.mapper;

import org.artifactory.mapper.ArtifactoryCentralMapperConfig;
import org.artifactory.storage.db.event.service.metadata.model.MutableArtifactMetadata;
import org.artifactory.storage.db.event.service.metadata.model.MutableMetadataEntityBOM;
import org.jfrog.metadata.client.model.MdsFile;
import org.jfrog.metadata.client.model.MdsPackage;
import org.jfrog.metadata.client.model.MdsPackageImpl;
import org.jfrog.metadata.client.model.MdsVersion;
import org.jfrog.metadata.client.rest.MdsPackageProxyFactory;
import org.mapstruct.*;

/**
 * @author Uriah Levy
 * One-way mapper from the {@link MutableMetadataEntityBOM} to the various MDS entity models.
 */
@Mapper(config = ArtifactoryCentralMapperConfig.class, nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS,
        uses = MdsEntityTranslator.class, componentModel = "spring", unmappedTargetPolicy = ReportingPolicy.IGNORE)
public interface MetadataEntityMapper {

    @ObjectFactory
    default MdsPackage createMdsPackage(@Context MdsPackage mdsPackage) {
        return MdsPackageProxyFactory.create(mdsPackage);
    }

    // Full mapper
    @Mapping(target = "created", qualifiedByName = {"ArtifactoryBaseMapper", "EpochToIso8601"})
    @Mapping(target = "modified", qualifiedByName = {"ArtifactoryBaseMapper", "EpochToIso8601"})
    @Mapping(target = "licenses", qualifiedByName = {"MdsEntityTranslator", "LicensesNamesToMdsLicense"})
    @Mapping(target = "stats", ignore = true)
    @Mapping(target = "qualifiers", ignore = true)
    @Mapping(target = "userProperties", ignore = true)
    MdsPackageImpl metadataBomToMdsPackage(MutableMetadataEntityBOM metadataBom);

    // Update (patch) mapper
    @Mapping(target = "created", qualifiedByName = {"ArtifactoryBaseMapper", "EpochToIso8601"})
    @Mapping(target = "modified", qualifiedByName = {"ArtifactoryBaseMapper", "EpochToIso8601"})
    @Mapping(target = "licenses", qualifiedByName = {"MdsEntityTranslator", "LicensesNamesToMdsLicense"})
    @Mapping(target = "stats", ignore = true)
    @Mapping(target = "qualifiers", ignore = true)
    @Mapping(target = "userProperties", ignore = true)
    MdsPackage metadataBomToMdsPackage(MutableMetadataEntityBOM metadataBom, @Context MdsPackage mdsPackageProxy);

    @Mapping(target = "created", qualifiedByName = {"ArtifactoryBaseMapper", "EpochToIso8601"})
    @Mapping(target = "licenses", qualifiedByName = {"MdsEntityTranslator", "LicensesNamesToMdsLicense"})
    @Mapping(target = "userProperties", qualifiedByName = {"MdsEntityTranslator", "UserPropertiesTranslator"})
    @Mapping(source = "downloadCount", target = "stats", qualifiedByName = {"MdsEntityTranslator", "StatsTranslator"})
    @Mapping(target = "qualifiers", qualifiedByName = {"MdsEntityTranslator", "QualifiersTranslator"})
    @Mapping(source = "version", target = "name")
    MdsVersion metadataBomToMdsVersion(MutableMetadataEntityBOM metadataBom);

    @Mapping(source = "artifactName", target = "name")
    @Mapping(source = "artifactMimeType", target = "mimeType")
    @Mapping(source = "artifactLength", target = "length")
    MdsFile metadataBomToMdsFile(MutableArtifactMetadata mutableArtifactMetadata);
}
