package org.artifactory.storage.db.binstore.service.garbage;

import com.google.common.collect.ImmutableList;
import org.artifactory.api.security.SecurityService;
import org.artifactory.storage.GCCandidate;
import org.artifactory.storage.binstore.service.GCProvider;
import org.artifactory.storage.binstore.service.GarbageCollectorStrategy;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.Mockito;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.util.Collections;
import java.util.concurrent.ThreadPoolExecutor;

import static org.mockito.Mockito.*;

/**
 * @author dudim
 */
@Test
public class GCCandidateProducerTest extends ArtifactoryHomeBoundTest {

    public void testStartAndStopAndStartAgain() throws InterruptedException {
        SecurityService securityService = mock(SecurityService.class);
        GCProvider gcProvider = mock(TrashAndBinariesGCProvider.class);
        GCCandidateProducer gcCandidateProducer = Mockito.spy(
                new GCCandidateProducer(Collections.singletonList(gcProvider),
                        GarbageCollectorStrategy.TRASH, securityService));

        when(gcProvider.getBatch()).thenReturn(ImmutableList.of(
                new GCCandidate(null, null, null, null, 0)));

        when(gcProvider.getAction()).thenReturn((o, o2) -> {
            System.out.println("Action is performed");
        });

        gcCandidateProducer.startProducer();
        verify(gcCandidateProducer, timeout(10_000)).run();
        verify(gcProvider, timeout(10_000).atLeastOnce()).getBatch();

        gcCandidateProducer.askToStopProducer();
        waitUntilProducerStopped(gcCandidateProducer);

        gcCandidateProducer.startProducer();
        verify(gcCandidateProducer, timeout(10_000).times(2)).run();
    }

    private void waitUntilProducerStopped(GCCandidateProducer gcCandidateProducer) throws InterruptedException {
        for (int i = 1; i <= 10; i++) {
            int activeThread = ((ThreadPoolExecutor) gcCandidateProducer.producerExecutor).getActiveCount();
            if (activeThread == 0) {
                break;
            }
            Thread.sleep(500);
            if (i == 10) {
                Assert.fail("Failed while waiting for producer to stop");
            }
        }
    }
}