package org.artifactory.storage.db.binstore.service.garbage;

import com.google.common.collect.ImmutableList;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.repo.trash.TrashService;
import org.artifactory.storage.GCCandidate;
import org.artifactory.storage.binstore.service.BinaryService;
import org.artifactory.storage.binstore.service.GCProvider;
import org.artifactory.storage.binstore.service.GarbageCollectorInfo;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import java.util.List;
import java.util.function.BiConsumer;

@Test
public class TrashAndBinariesGCProviderTest {

    @Mock
    private RepositoryService repositoryService;
    @Mock
    private TrashService trashService;
    @Mock
    private BinaryService binaryService;

    @BeforeClass
    private void setup() {
        MockitoAnnotations.initMocks(this);
    }

    public void testGetAction() {
        GCProvider provider = new TrashAndBinariesGCProvider(repositoryService, trashService, binaryService);
        BiConsumer<GCCandidate, GarbageCollectorInfo> cleanupAction = provider.getAction();

        GarbageCollectorInfo gcInfo = new GarbageCollectorInfo();
        List<GCCandidate> candidatesForDeletion = ImmutableList.of(
                new GCCandidate(null, "aaa", "aaa", "aaa", 55),
                new GCCandidate(null, "bbb", "bbb", "bbb", 55),
                new GCCandidate(null, "ccc", "ccc", "ccc", 55)
        );

        Mockito.when(repositoryService.existsBySha1("aaa")).thenReturn(true);
        Mockito.when(repositoryService.existsBySha1("bbb")).thenReturn(true);
        Mockito.when(repositoryService.existsBySha1("ccc")).thenReturn(false);

        candidatesForDeletion.forEach(candidate -> {
            cleanupAction.accept(candidate, gcInfo);
            Mockito.verify(trashService).undeployFromTrash(candidate);
            Mockito.verify(repositoryService).existsBySha1(candidate.getSha1());
        });

        Mockito.verify(binaryService).executeBinaryCleaner(candidatesForDeletion.get(2), gcInfo);
    }
}