package org.artifactory.storage.db.binstore.service.garbage;

import com.google.common.collect.ImmutableList;
import org.artifactory.repo.trash.TrashService;
import org.artifactory.storage.GCCandidate;
import org.artifactory.storage.binstore.service.GCProvider;
import org.artifactory.storage.binstore.service.GarbageCollectorInfo;
import org.artifactory.storage.db.fs.dao.NodesDao;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import java.util.List;
import java.util.function.BiConsumer;

import static org.mockito.Mockito.doNothing;

/**
 * @author Uriah Levy
 */
@Test
public class TrashGCProviderTest extends ArtifactoryHomeBoundTest {

    @Mock
    private TrashService trashService;
    @Mock
    private NodesDao nodesDao;

    @BeforeClass
    private void setup() {
        MockitoAnnotations.initMocks(this);
    }

    public void testGetAction() {
        GCProvider provider = new TrashGCProvider(trashService, nodesDao);
        BiConsumer<GCCandidate, GarbageCollectorInfo> cleanupAction = provider.getAction();

        GarbageCollectorInfo gcInfo = new GarbageCollectorInfo();
        List<GCCandidate> candidatesForDeletion = ImmutableList.of(
                new GCCandidate(null, "aaa", "aaa", "aaa", 55),
                new GCCandidate(null, "bbb", "bbb", "bbb", 55),
                new GCCandidate(null, "ccc", "ccc", "ccc", 55)
        );

        doNothing().when(trashService).undeployFromTrash(new GCCandidate(null, "aaa", "aaa", "aaa", 55));
        doNothing().when(trashService).undeployFromTrash(new GCCandidate(null, "bbb", "bbb", "bbb", 55));
        doNothing().when(trashService).undeployFromTrash(new GCCandidate(null, "ccc", "ccc", "ccc", 55));

        candidatesForDeletion.forEach(candidate -> {
            cleanupAction.accept(candidate, gcInfo);
            Mockito.verify(trashService).undeployFromTrash(candidate);
        });
    }
}