package org.artifactory.storage.db.event.itest.dao;

import org.artifactory.storage.db.event.dao.NodeEventPrioritiesDao;
import org.artifactory.storage.db.event.entity.DbPrioritizedNodeEvent;
import org.artifactory.storage.db.itest.DbBaseTest;
import org.jfrog.common.ClockUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import java.sql.SQLException;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

/**
 * @author Uriah Levy
 */
@Test
public class NodeEventPrioritiesDaoTest extends DbBaseTest {
    @Autowired
    private NodeEventPrioritiesDao dao;

    @BeforeClass
    public void setup() {
        importSql("/sql/nodes.sql");
    }

    public void testGetEventByPriority() throws SQLException {
        assertEquals(dao.getEventsByPriorityAndOperatorId(2, "metadata-dispatcher").size(), 2);
        assertEquals(dao.getEventsByPriorityAndOperatorId(3, "metadata-dispatcher").size(), 2);
    }

    public void testInsertEventPriority() throws SQLException {
        dao.insertEventPriority(getNodeEventPriority());
        assertEquals(dao.getEventsByPriorityAndOperatorId(2, "metadata-dispatcher").size(), 3);
    }

    @Test(expectedExceptions = SQLException.class)
    public void testPrimaryKeyConstraintViolated() throws SQLException {
        dao.insertEventPriority(getNodeEventPriority());
        assertEquals(dao.getEventsByPriorityAndOperatorId(2, "metadata-dispatcher").size(), 5);
    }

    @Test(dependsOnMethods = "testInsertEventPriority")
    public void deleteEventPriorityById() throws SQLException {
        assertEquals(dao.getEventsByPriorityAndOperatorId(2, "metadata-dispatcher").size(), 3);
        dao.deleteEventPriorityById(getNodeEventPriority().getPriorityId());
        assertEquals(dao.getAll().size(), 4);
        assertEquals(dao.getEventsByPriorityAndOperatorId(2, "metadata-dispatcher").size(), 2);
    }

    @Test(dependsOnMethods = {"testInsertEventPriority"})
    public void testCountEventsByOperatorId() throws SQLException {
        assertTrue(dao.countRetryableEventsByOperatorId("metadata-dispatcher").isPresent());
        assertEquals(dao.countRetryableEventsByOperatorId("metadata-dispatcher").get().intValue(), 3);
    }

    @Test(dependsOnMethods = "testInsertEventPriority")
    public void testGetMinimalPriorityByOperatorId() throws SQLException {
        assertTrue(dao.getMinimalPriorityByOperatorId("metadata-dispatcher").isPresent());
        assertEquals(dao.getMinimalPriorityByOperatorId("metadata-dispatcher").get().intValue(), 2);
        DbPrioritizedNodeEvent nodeEventPriority = getNodeEventPriority();
        nodeEventPriority.setPriority(1);
        nodeEventPriority.setPriorityId(1234567891L);
        dao.insertEventPriority(nodeEventPriority);
        assertEquals(dao.getMinimalPriorityByOperatorId("metadata-dispatcher").get().intValue(), 1);
        dao.deleteAll();
        // no values -> minimal priority is 1
        assertTrue(dao.getAll().isEmpty());
        assertEquals(dao.getMinimalPriorityByOperatorId("metadata-dispatcher").get().intValue(), 0);
    }

    private DbPrioritizedNodeEvent getNodeEventPriority() {
        return new DbPrioritizedNodeEvent(123456789, "a/b/c", "create", "metadata-dispatcher", 2, ClockUtils.epochMillis(), 0);
    }
}
