package org.artifactory.storage.db.event.itest.dao;

import org.artifactory.storage.db.event.dao.NodeEventsCursorDao;
import org.artifactory.storage.db.event.entity.DbNodeEventCursor;
import org.artifactory.storage.db.itest.DbBaseTest;
import org.jfrog.common.ClockUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import java.sql.SQLException;
import java.util.Optional;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

/**
 * @author Uriah Levy
 */
@Test
public class NodeEventsCursorDaoTest extends DbBaseTest {

    @Autowired
    private NodeEventsCursorDao dao;

    @BeforeClass
    public void setup() {
        importSql("/sql/nodes.sql");
    }

    public void allEvents() throws SQLException {
        assertEquals(dao.getAll().size(), 2);
    }

    public void cursorForOperationId() throws SQLException {
        Optional<DbNodeEventCursor> dbNodeEventCursor = dao.cursorForOperator("metadata-dispatcher");
        assertTrue(dbNodeEventCursor.isPresent());
        assertEquals(dbNodeEventCursor.get().getEventMarker(), 1515653250000L);
    }

    @Test(dependsOnMethods = "allEvents")
    public void createCursor() throws SQLException {
        String operatorId = "replication-ams-nj4";
        long now = ClockUtils.epochMillis();
        dao.insert(DbNodeEventCursor.builder()
                .operatorId(operatorId)
                .eventMarker(now)
                .build());
        assertEquals(dao.getAll().size(), 3);
        Optional<DbNodeEventCursor> cursor = dao.cursorForOperator(operatorId);
        assertTrue(cursor.isPresent());
        assertEquals(cursor.get().getEventMarker(), now);
    }

    @Test(dependsOnMethods = {"createCursor"}, expectedExceptions = SQLException.class)
    public void createExistingCursor() throws SQLException {
        String operatorId = "replication-ams-nj4";
        final DbNodeEventCursor cursor = DbNodeEventCursor.builder()
                .operatorId(operatorId)
                .eventMarker(ClockUtils.epochMillis())
                .build();
        // primary key violation
        dao.insert(cursor);
    }
}
