package org.artifactory.ui.rest.service.admin.security.permissions;

import com.google.common.collect.Lists;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.artifactory.rest.common.service.security.permissions.RestSecurityRequestHandlerV2;
import org.artifactory.security.permissions.PermissionTargetModel;
import org.artifactory.ui.rest.model.admin.security.permissions.AllPermissionTargetsUIModel;
import org.artifactory.ui.rest.model.utils.repositories.RepoKeyType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

import static org.artifactory.repo.RepoDetailsType.*;
import static org.artifactory.security.PermissionTargetNaming.NAMING_UI;

/**
 * Used in main Permissions page
 *
 * @author Dan Feldman
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class GetAllPermissionTargetsService implements RestService {

    @Autowired
    private CentralConfigService configService;

    @Autowired
    private RestSecurityRequestHandlerV2 securityRequestHandler;

    @Override
    public void execute(ArtifactoryRestRequest request, RestResponse response) {
        List<RepoKeyType> allRealRepos = getAllRepoKeys();
        //These are all of the permission targets the current user has permission to manage
        List<PermissionTargetModel> allPermissionTargets = securityRequestHandler.getAllPermissionTargets(NAMING_UI);
        List<AllPermissionTargetsUIModel> modelList = allPermissionTargets.stream()
                .map(permissionTarget -> new AllPermissionTargetsUIModel(permissionTarget, allRealRepos))
                .collect(Collectors.toList());
        response.iModel(modelList);
    }

    /**
     * @return list of all current repo keys.
     */
    private List<RepoKeyType> getAllRepoKeys() {
        List<RepoKeyType> repos = Lists.newArrayList();
        configService.getDescriptor().getLocalRepositoriesMap().values().stream()
                .filter(repo -> !RepoType.BuildInfo.equals(repo.getType()))
                .map(repo -> new RepoKeyType(LOCAL.typeNameLowercase(), repo.getType(), repo.getKey()))
                .forEach(repos::add);
        configService.getDescriptor()
                .getRemoteRepositoriesMap()
                .values()
                .stream()
                .map(repo ->
                        new RepoKeyType(REMOTE.typeNameLowercase(), repo.getType(), repo.getKey()))
                .forEach(repos::add);
        configService.getDescriptor().getDistributionRepositoriesMap().values().stream()
                .map(repo -> new RepoKeyType(DISTRIBUTION.typeNameLowercase(), repo.getType(), repo.getKey()))
                .forEach(repos::add);
        return repos;
    }
}
