/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.rest.service.builds.buildsinfo;

import org.artifactory.api.build.BuildService;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.build.BuildId;
import org.artifactory.common.ConstantValues;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.artifactory.ui.rest.model.artifacts.browse.treebrowser.tabs.permission.PagingModel;
import org.artifactory.ui.rest.model.builds.GeneralBuildInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author Chen Keinan
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class GetAllBuildIDsService implements RestService {

    @Autowired
    private CentralConfigService centralConfigService;

    @Autowired
    private BuildService buildService;

    @Autowired
    private AuthorizationService authService;

    @Override
    public void execute(ArtifactoryRestRequest request, RestResponse response) {
        // fetch build info data
        fetchAllBuildsData(response);
    }

    /**
     * fect all build data by type
     *
     * @param artifactoryResponse - encapsulate data related to response
     */
    private void fetchAllBuildsData(RestResponse artifactoryResponse) {
        List<GeneralBuildInfo> generalBuildInfoList = buildService.getLatestBuildIDsPaging()
                .stream()
                .map(this::getBuildGeneralInfo)
                .collect(Collectors.toList());
        PagingModel pagingModel = new PagingModel(0, generalBuildInfoList);
        artifactoryResponse.iModel(pagingModel);
    }

    private GeneralBuildInfo getBuildGeneralInfo(BuildId build) {
                return GeneralBuildInfo.builder()
                        .buildName(build.getName())
                        .lastBuildTime(centralConfigService.getDateFormatter().print(build.getStartedDate().getTime()))
                        .buildNumber(build.getNumber())
                        .time(build.getStartedDate() != null ? build.getStartedDate().getTime() : 0)
                        .canDelete(ConstantValues.buildUiSkipDeletePermissionCheck.getBoolean() ||
                                authService.canDeleteBuild(build.getName(), build.getNumber(), build.getStarted()))
                        .build();
    }
}
