/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.rest.service.onboarding;

import org.artifactory.addon.AddonsManager;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.security.SecurityService;
import org.artifactory.api.security.UserGroupService;
import org.artifactory.common.ConstantValues;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.artifactory.security.UserInfo;
import org.artifactory.ui.rest.model.onboarding.ArtifactoryInitStatusModel;
import org.jfrog.security.util.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.Optional;

import static org.artifactory.security.FirstStartupHelper.onlyDefaultReposExist;

/**
 * @author nadavy
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class GetArtifactoryInitStatusService implements RestService {

    private SecurityService securityService;
    private AddonsManager addonsManager;
    private CentralConfigService configService;
    private UserGroupService userGroupService;

    @Autowired
    public GetArtifactoryInitStatusService(SecurityService securityService, AddonsManager addonsManager,
            CentralConfigService configService, UserGroupService userGroupService) {
        this.securityService = securityService;
        this.addonsManager = addonsManager;
        this.configService = configService;
        this.userGroupService = userGroupService;
    }

    public void execute(ArtifactoryRestRequest request, RestResponse response) {
        boolean hasLicenseAlready = addonsManager.isLicenseInstalled();

        long lastLoginTime;
        UserInfo userInfo = userGroupService.currentUser();
        if (userInfo != null && userInfo.getUsername().equals("admin")) {
            lastLoginTime = userInfo.getLastLoginTimeMillis();
        } else {
            lastLoginTime = Optional.ofNullable(securityService.getUserLastLoginInfo("admin"))
                    .map(Pair::getSecond)
                    .orElse(0L);
        }

        boolean hasPriorLogins = lastLoginTime != 0;

        boolean skipWizard = ConstantValues.skipOnboardingWizard.getBoolean();
        boolean hasProxies = !configService.getDescriptor().getProxies().isEmpty();
        ArtifactoryInitStatusModel artifactoryInitStatusModel = new ArtifactoryInitStatusModel(
                onlyDefaultReposExist(configService.getDescriptor()), hasLicenseAlready, hasPriorLogins, hasProxies, skipWizard);
        response.iModel(artifactoryInitStatusModel);
    }
}
